#include "gbfs.h"


//---------------------------------------------------------------------------
// gba_cart.ld
extern u8 __rom_end__;

ST_GBFS Gbfs;

//---------------------------------------------------------------------------
EWRAM_CODE void GbfsInit()
{
	_Memset((u8*)&Gbfs, 0x00, sizeof(ST_GBFS));


	char* pHeader = (char*)(((u32)&__rom_end__ + 0xff) & 0xffffff00);

	if(_Strncmp(pHeader, "PinEightGBFS", sizeof("PinEightGBFS")-1) == 0)
	{
		Gbfs.pHeader = (ST_GBFS_HEADER*)pHeader;
		Gbfs.pList   = (ST_GBFS_LIST*)(pHeader + Gbfs.pHeader->dirOff);
	}
	else
	{
		SystemError("[Err][GbfsInit] .gbfs File Not Found");
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void* GBFSGetPointer(char* fname)
{
	u16 i;

	for(i=0; i<Gbfs.pHeader->fileCnt; i++)
	{
		if(_Strncmp(fname, Gbfs.pList[i].fname, 24) == 0)
		{
			Gbfs.size = Gbfs.pList[i].size;
			return (u8*)Gbfs.pHeader + Gbfs.pList[i].dataOff;
		}
	}

	return NULL;
}
//---------------------------------------------------------------------------
IWRAM_CODE void* GBFSGetPointer2(u32 cnt)
{
	if(cnt >= Gbfs.pHeader->fileCnt)
	{
		return NULL;
	}

	Gbfs.size = Gbfs.pList[cnt].size;
	return (u8*)Gbfs.pHeader + Gbfs.pList[cnt].dataOff;
}
//---------------------------------------------------------------------------
IWRAM_CODE void* GBFSGetSafePointer(char* fname)
{
	void* p = GBFSGetPointer(fname);

	if(p == NULL)
	{
		SystemError("[Err][GBFSGetSafePointer] .gbfs File Not Found: %s\n", fname);
	}

	return p;
}
//---------------------------------------------------------------------------
IWRAM_CODE void* GBFSGetSafePointer2(u32 cnt)
{
	void* p = GBFSGetPointer2(cnt);

	if(p == NULL)
	{
		SystemError("[Err][GBFSGetSafePointer2] .gbfs File Not Found: No.%d\n", cnt);
	}

	return p;
}
//---------------------------------------------------------------------------
IWRAM_CODE u32 GbfsGetBodySize()
{
	return Gbfs.size;
}
//---------------------------------------------------------------------------
IWRAM_CODE u16 GbfsGetCnt()
{
	return Gbfs.pHeader->fileCnt;
}
//---------------------------------------------------------------------------
IWRAM_CODE u32 GbfsGetSize()
{
	return Gbfs.pHeader->size;
}
